"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Irc = void 0;
const n8n_workflow_1 = require("n8n-workflow");
const IrcClient_1 = require("./IrcClient");
const IrcParser_1 = require("./IrcParser");
class Irc {
    constructor() {
        this.description = {
            displayName: ' Internet Relay Chat (IRC)',
            name: 'irc',
            icon: 'file:irc.svg',
            group: ['output'],
            version: 1,
            description: 'Sends data to an IRC channel',
            subtitle: '={{$parameter["operation"] + ": " + $parameter["resource"]}}',
            defaults: {
                name: 'IRC',
            },
            inputs: ['main'],
            outputs: ['main'],
            credentials: [
                {
                    name: 'ircApi',
                    required: true,
                },
            ],
            properties: [
                {
                    displayName: 'Resource',
                    name: 'resource',
                    noDataExpression: true,
                    type: 'options',
                    options: [
                        {
                            name: 'Message',
                            value: 'message',
                        },
                    ],
                    default: 'message',
                },
                {
                    displayName: 'Operation',
                    name: 'operation',
                    noDataExpression: true,
                    type: 'options',
                    displayOptions: {
                        show: {
                            resource: [
                                'message',
                            ],
                        },
                    },
                    options: [
                        {
                            name: 'Create',
                            value: 'create',
                            action: 'Create a message',
                            description: 'Send a message to a channel',
                        },
                    ],
                    default: 'create',
                },
                {
                    displayName: 'Channel Name',
                    name: 'channelName',
                    type: 'string',
                    required: true,
                    default: '',
                    description: 'The channel to send the message to',
                },
                {
                    displayName: 'Text',
                    name: 'text',
                    type: 'string',
                    default: '',
                    placeholder: 'Hello from n8n!',
                    description: 'The text to send',
                },
                {
                    displayName: 'Join Channel',
                    name: 'joinChannel',
                    type: 'boolean',
                    default: true,
                    description: 'Whether or not to join the channel',
                },
                {
                    displayName: 'Channel Key',
                    name: 'channelKey',
                    type: 'string',
                    default: '',
                    displayOptions: {
                        show: {
                            'joinChannel': [
                                true,
                            ],
                        },
                    },
                    description: 'The key used to join this channel (optional)',
                },
                {
                    displayName: 'Message Type',
                    name: 'messageType',
                    type: 'options',
                    options: [
                        {
                            name: 'Emote',
                            value: 'action',
                            description: 'Perform an action (/me)',
                        },
                        {
                            name: 'Notice',
                            value: 'notice',
                            description: 'Send a notice',
                        },
                        {
                            name: 'Privmsg',
                            value: 'privmsg',
                            description: 'Send a privmsg',
                        },
                    ],
                    default: 'privmsg',
                    description: 'The type of message to send',
                },
                {
                    displayName: 'Output Raw Logs',
                    name: 'outputRawLogs',
                    type: 'boolean',
                    default: false,
                    description: 'Whether to output log of raw IRC traffic',
                },
            ],
        };
    }
    async execute() {
        const outputInfo = {};
        const items = this.getInputData();
        const resource = this.getNodeParameter('resource', 0);
        const operation = this.getNodeParameter('operation', 0);
        const credentials = await this.getCredentials('ircApi');
        if (credentials === undefined) {
            throw new n8n_workflow_1.NodeOperationError(this.getNode(), 'No credentials got returned!');
        }
        if (resource === 'message') {
            if (operation === 'create') {
                const desiredNick = credentials.nickname;
                const ident = credentials.ident;
                const realname = credentials.realname;
                const serverPassword = credentials.serverPassword;
                const outputRawLogs = this.getNodeParameter('outputRawLogs', 0);
                const client = new IrcClient_1.IrcClient(desiredNick, ident, realname, outputRawLogs);
                if (credentials.saslType === 'plain') {
                    const saslUsername = credentials.saslAccountName;
                    const saslPassword = credentials.saslAccountPassword;
                    const saslRequired = credentials.saslRequired;
                    client.setupSaslPlain(saslUsername, saslPassword, saslRequired);
                }
                const messageType = this.getNodeParameter('messageType', 0);
                const channelName = (0, IrcParser_1.EnsureIrcParam)(this.getNodeParameter('channelName', 0));
                const joinChannel = this.getNodeParameter('joinChannel', 0);
                let channelKey = this.getNodeParameter('channelKey', 0);
                if (joinChannel && channelKey) {
                    channelKey = (0, IrcParser_1.EnsureIrcParam)(channelKey);
                }
                const sentLines = [];
                if (credentials.tls) {
                    const tlsConnectionOptions = {
                        port: credentials.tlsPort,
                        host: credentials.host,
                        rejectUnauthorized: credentials.tlsValidation,
                    };
                    if (tlsConnectionOptions.rejectUnauthorized) {
                        tlsConnectionOptions.servername = tlsConnectionOptions.host;
                    }
                    client.connect(undefined, tlsConnectionOptions, serverPassword);
                }
                else {
                    const netConnectionOptions = {
                        port: credentials.plainPort,
                        host: credentials.host,
                    };
                    client.connect(netConnectionOptions, undefined, serverPassword);
                }
                client.on('connected', () => {
                    if (joinChannel) {
                        client.send('JOIN', channelName, channelKey);
                    }
                    let verb = 'PRIVMSG';
                    if (messageType === 'notice') {
                        verb = 'NOTICE';
                    }
                    for (let i = 0; i < items.length; i++) {
                        const text = this.getNodeParameter('text', i);
                        text.split('\n').forEach(line => {
                            const subLines = [];
                            while (line.length > 410) {
                                subLines.push(line.slice(0, 410));
                                line = line.slice(410);
                            }
                            subLines.push(line);
                            subLines.forEach(subLine => {
                                sentLines.push(subLine);
                                if (messageType === 'action') {
                                    subLine = `\x01ACTION ${subLine}\x01`;
                                }
                                client.send(verb, channelName, subLine);
                            });
                        });
                    }
                    client.send('QUIT');
                });
                await client.runUntilClosed();
                const statusInfo = client.statusInfo();
                if (statusInfo.error) {
                    if (this.continueOnFail()) {
                        outputInfo.error = statusInfo.error;
                    }
                    else {
                        throw new n8n_workflow_1.NodeApiError(this.getNode(), statusInfo);
                    }
                }
                outputInfo.account = client.account;
                outputInfo.nick = client.nick;
                outputInfo.sentLines = sentLines;
                outputInfo.timesNickWasInUse = client.timesNickWasInUse;
                if (outputRawLogs) {
                    outputInfo.log = statusInfo.log;
                }
            }
        }
        return [this.helpers.returnJsonArray(outputInfo)];
    }
}
exports.Irc = Irc;
//# sourceMappingURL=Irc.node.js.map