"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.IrcClient = void 0;
const events_1 = require("events");
const net = require("net");
const tls = require("tls");
const IrcParser_1 = require("./IrcParser");
class IrcClient extends events_1.EventEmitter {
    constructor(nick, ident, realname, saveRawLogs) {
        super();
        this.nick = nick;
        this.ident = ident;
        this.realname = realname;
        this.saveRawLogs = saveRawLogs;
        this.usingTLS = false;
        this.bufferedData = '';
        this.rawLog = '';
        this.errorMessage = '';
        this.timesNickWasInUse = 0;
        this.connectionRegComplete = false;
        this.saslPlainUsername = '';
        this.saslPlainPassword = '';
        this.saslRequired = false;
        this.account = '';
        this.nick = (0, IrcParser_1.EnsureIrcNick)(this.nick);
        this.ident = (0, IrcParser_1.EnsureIrcParam)(this.ident);
        this.realname = (0, IrcParser_1.EnsureIrcFinalParam)(this.realname);
        this.on('irc authenticate', this.handleAuthenticate);
        this.on('irc cap', this.handleCap);
        this.on('irc ping', this.handlePing);
        this.on('irc 001', this.handleWelcome);
        this.on('irc 005', this.handleIsupport);
        this.on('irc 376', this.handleRegComplete);
        this.on('irc 401', this.handleCannotSendMessage);
        this.on('irc 403', this.handleCannotJoinChannel);
        this.on('irc 404', this.handleCannotSendMessage);
        this.on('irc 422', this.handleRegComplete);
        this.on('irc 432', this.handleBadNick);
        this.on('irc 433', this.handleNickInUse);
        this.on('irc 475', this.handleCannotJoinChannel);
        this.on('irc 900', this.handleLoggedIn);
        this.on('irc 903', this.handleSaslSuccess);
        this.on('irc 904', this.handleSaslFail);
    }
    connect(netConnectionOptions, tlsConnectionOptions, serverPassword) {
        this.serverPassword = serverPassword;
        if (netConnectionOptions && tlsConnectionOptions) {
            throw new Error('You can only send either netConnectionOptions or tlsConnectionOptions, not both');
        }
        else if (tlsConnectionOptions) {
            this.socket = tls.connect(tlsConnectionOptions, () => {
                this.socketConnected();
            });
            this.usingTLS = true;
        }
        else if (netConnectionOptions) {
            this.socket = net.connect(netConnectionOptions, () => {
                this.socketConnected();
            });
        }
        else {
            throw new Error('You must send either netConnectionOptions or tlsConnectionOptions');
        }
        this.socket.setEncoding('utf8');
        this.socket.on('data', this.socketData.bind(this));
        this.socket.on('close', this.socketClosed.bind(this));
        this.socket.on('error', this.socketError.bind(this));
    }
    socketConnected() {
        if (this.serverPassword) {
            this.send('PASS', this.serverPassword);
        }
        if (this.saslPlainUsername || this.saslRequired) {
            this.send('CAP', 'REQ', 'sasl');
        }
        this.send('NICK', this.nick);
        this.send('USER', this.ident, '0', '*', this.realname);
    }
    socketData(input) {
        if (input instanceof Buffer) {
            input = input.toString();
        }
        this.bufferedData += input;
        const messages = this.bufferedData.split('\n');
        this.bufferedData = messages.pop();
        messages.forEach(msgString => {
            msgString = msgString.replace(/\r/g, '');
            if (this.saveRawLogs) {
                this.rawLog += `<-  ${msgString}\n`;
            }
            const message = (0, IrcParser_1.ParseIrcMessage)(msgString);
            this.emit(`irc ${message.verb.toLowerCase()}`, message);
        });
    }
    socketClosed(hadError) {
        this.emit('closed', hadError);
    }
    socketError(err) {
        this.errorMessage = `socket error: ${err.message}`;
    }
    handleAuthenticate(message) {
        if (message.params && message.params[0] === '+') {
            let saslBlob = Buffer.from(`${this.saslPlainUsername}\x00${this.saslPlainUsername}\x00${this.saslPlainPassword}`).toString('base64');
            while (saslBlob) {
                const tmp = saslBlob.slice(0, 400);
                this.send('AUTHENTICATE', tmp);
                saslBlob = saslBlob.slice(400);
                if (tmp.length === 400 && !saslBlob) {
                    this.send('AUTHENTICATE', '+');
                }
            }
        }
        else {
            this.errorMessage = `SASL handleAuthenticate failed, we unexpectedly got [${message.toString()}]`;
            this.send('QUIT');
        }
    }
    handleCap(message) {
        if (message.params.length > 2 && message.params[1].toLowerCase() === 'ack') {
            this.send('AUTHENTICATE', 'PLAIN');
        }
        else if (this.saslRequired) {
            this.errorMessage = 'SASL Login failed, could not request capability (maybe services is down)';
            this.send('QUIT');
        }
        else {
            this.send('CAP', 'END');
        }
    }
    handlePing(message) {
        this.send('PONG', ...message.params);
    }
    handleWelcome(message) {
        this.nick = message.params[0];
    }
    handleIsupport(message) {
        if (this.connectionRegComplete) {
            return;
        }
        message.params.forEach((param, i) => {
            if (i < message.params.length - 1) {
                const split = param.split('=');
                if (split.length === 2 && split[0].toLowerCase() === 'bot') {
                    this.send('MODE', this.nick, `+${split[1]}`);
                }
            }
        });
    }
    handleRegComplete(message) {
        if (!this.connectionRegComplete) {
            this.connectionRegComplete = true;
            this.emit('connected');
        }
    }
    handleCannotSendMessage(message) {
        if (!this.errorMessage) {
            this.errorMessage = `Could not send message: ${message.finalParam()}`;
        }
    }
    handleBadNick(message) {
        this.errorMessage = `Nickname is not valid: ${message.finalParam()}`;
        this.send('QUIT');
    }
    handleNickInUse(message) {
        if (!this.connectionRegComplete) {
            this.timesNickWasInUse += 1;
            if (this.timesNickWasInUse > 5) {
                this.errorMessage = 'Nick was in use';
                this.send('QUIT');
                return;
            }
            this.send('NICK', message.params[1] + '_');
        }
    }
    handleCannotJoinChannel(message) {
        this.errorMessage = `Could not join channel: ${message.finalParam()}`;
    }
    handleLoggedIn(message) {
        if (message.params.length > 2) {
            this.account = message.params[2];
        }
    }
    handleSaslSuccess(message) {
        this.send('CAP', 'END');
    }
    handleSaslFail(message) {
        if (this.saslRequired) {
            this.errorMessage = `[904] ${message.finalParam()}`;
            this.send('QUIT');
        }
        else {
            this.send('CAP', 'END');
        }
    }
    setupSaslPlain(username, password, required) {
        this.saslPlainUsername = username;
        this.saslPlainPassword = password;
        if (required !== undefined) {
            this.saslRequired = required;
        }
    }
    sendLine(input) {
        if (this.socket === undefined) {
            return;
        }
        this.socket.write(`${input}\r\n`);
        if (this.saveRawLogs) {
            this.rawLog += ` -> ${input}\n`;
        }
    }
    send(verb, ...params) {
        this.sendLine(new IrcParser_1.IrcMessage('', verb, params).toString());
    }
    statusInfo() {
        return {
            'type': 'IrcStatus',
            'error': this.errorMessage,
            'log': this.rawLog,
        };
    }
    async runUntilClosed() {
        if (!this.socket || this.socket.destroyed) {
            return;
        }
        await (0, events_1.once)(this, 'closed');
    }
}
exports.IrcClient = IrcClient;
//# sourceMappingURL=IrcClient.js.map