"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ParseIrcMessage = exports.IrcMessage = exports.EnsureIrcFinalParam = exports.EnsureIrcParam = exports.EnsureIrcNick = void 0;
function EnsureIrcNick(nick) {
    nick = EnsureIrcParam(nick);
    if (nick.includes(',') || nick.includes('*') || nick.includes('?') ||
        nick.includes('.') || nick.includes('!') || nick.includes('@')) {
        throw new Error(`Nickname [${nick}] can't include [,*?.!@]`);
    }
    return nick;
}
exports.EnsureIrcNick = EnsureIrcNick;
function EnsureIrcParam(input) {
    input = input.trim();
    if (!input) {
        throw new Error(`Required property is empty`);
    }
    if (input.startsWith(':') || input.includes(' ') ||
        input.includes('\r') || input.includes('\n')) {
        throw new Error(`Property [${input}] can't start with a colon (:) or contain spaces/newlines`);
    }
    return input;
}
exports.EnsureIrcParam = EnsureIrcParam;
function EnsureIrcFinalParam(input) {
    input = input.trim();
    if (input.includes('\r') || input.includes('\n')) {
        throw new Error(`Property [${input}] can't contain newlines`);
    }
    return input;
}
exports.EnsureIrcFinalParam = EnsureIrcFinalParam;
class IrcMessage {
    constructor(prefix, verb, params) {
        this.prefix = prefix;
        this.verb = verb;
        this.params = params;
        this.ensureParts();
    }
    ensureParts() {
        if (this.prefix) {
            this.prefix = EnsureIrcParam(this.prefix);
        }
        this.verb = EnsureIrcParam(this.verb);
        this.params.forEach((param, i) => {
            if (i === this.params.length - 1) {
                EnsureIrcFinalParam(param);
            }
            else {
                EnsureIrcParam(param);
            }
        });
    }
    toString() {
        this.ensureParts();
        let messageParts = [];
        if (this.prefix) {
            messageParts.push(`:${this.prefix}`);
        }
        messageParts.push(this.verb);
        if (this.params) {
            messageParts = messageParts.concat(this.params);
            try {
                EnsureIrcParam(messageParts[messageParts.length - 1]);
            }
            catch {
                messageParts[messageParts.length - 1] = `:${messageParts[messageParts.length - 1]}`;
            }
        }
        return messageParts.join(' ');
    }
    finalParam() {
        return this.params ? this.params[this.params.length - 1] : '';
    }
}
exports.IrcMessage = IrcMessage;
function ParseIrcMessage(input) {
    input = input.trimLeft().replace(/[\r\n]+$/, '');
    if (input.includes('\r') || input.includes('\n')) {
        throw new Error('IRC message cannot be parsed, contains \\r\\n');
    }
    if (input.trim() === '') {
        throw new Error('IRC message cannot be parsed, is empty');
    }
    let i = 0;
    let j = 0;
    const invalidMessageError = Error('IRC message cannot be parsed, is invalid');
    let prefix = '';
    if (input[0] === ':') {
        j = i + 1;
        while (input[j] !== ' ') {
            j++;
            if (input.length <= j) {
                throw invalidMessageError;
            }
        }
        prefix = input.substring(i + 1, j);
        while (input[j] === ' ') {
            j++;
            if (input.length <= j) {
                throw invalidMessageError;
            }
        }
        i = j;
    }
    let verb = '';
    j = i;
    while (input[j] !== ' ') {
        j++;
        if (input.length <= j) {
            break;
        }
    }
    verb = input.substring(i, j);
    if (j < input.length) {
        while (input[j] === ' ') {
            j++;
            if (input.length <= j) {
                break;
            }
        }
    }
    i = j;
    const params = [];
    let trailingHit = false;
    while (i < input.length) {
        if (input[i] === ':') {
            trailingHit = true;
            params.push(input.substring(i + 1));
            break;
        }
        j = i;
        while (input[j] !== ' ') {
            j++;
            if (input.length <= j) {
                break;
            }
        }
        params.push(input.substring(i, j));
        if (j < input.length) {
            while (input[j] === ' ') {
                j++;
                if (input.length <= j) {
                    break;
                }
            }
        }
        i = j;
    }
    if (!trailingHit && input[i] === ' ') {
        params.push('');
    }
    return new IrcMessage(prefix, verb, params);
}
exports.ParseIrcMessage = ParseIrcMessage;
//# sourceMappingURL=IrcParser.js.map